@@ -75,24 +75,13 @@ set_wm_check_hint (MetaScreen *screen)
                    XA_WINDOW,
                    32, PropModeReplace, (guchar*) data, 1);
 
-  /* Legacy GNOME hint (uses cardinal, dunno why) */
-
-  /* do this after setting up window fully, to avoid races
-   * with clients listening to property notify on root.
-   */
-  XChangeProperty (screen->display->xdisplay, screen->xroot,
-                   screen->display->atom_win_supporting_wm_check,
-                   XA_CARDINAL,
-                   32, PropModeReplace, (guchar*) data, 1);
-  
   return Success;
 }
 
 static int
 set_supported_hint (MetaScreen *screen)
 {
-#define N_SUPPORTED 45
-#define N_WIN_SUPPORTED 1
+#define N_SUPPORTED 49
   Atom atoms[N_SUPPORTED];
   
   atoms[0] = screen->display->atom_net_wm_name;
@@ -116,43 +105,39 @@ set_supported_hint (MetaScreen *screen)
   atoms[18] = screen->display->atom_net_client_list_stacking;
   atoms[19] = screen->display->atom_net_wm_state_skip_taskbar;
   atoms[20] = screen->display->atom_net_wm_state_skip_pager;
-  atoms[21] = screen->display->atom_net_wm_icon;
-  atoms[22] = screen->display->atom_net_wm_moveresize;
-  atoms[23] = screen->display->atom_net_wm_state_hidden;
-  atoms[24] = screen->display->atom_net_wm_window_type_utility;
-  atoms[25] = screen->display->atom_net_wm_window_type_splash;
-  atoms[26] = screen->display->atom_net_wm_state_fullscreen;
-  atoms[27] = screen->display->atom_net_wm_ping;
-  atoms[28] = screen->display->atom_net_active_window;
-  atoms[29] = screen->display->atom_net_workarea;
-  atoms[30] = screen->display->atom_net_showing_desktop;
-  atoms[31] = screen->display->atom_net_desktop_layout;
-  atoms[32] = screen->display->atom_net_desktop_names;
-  atoms[33] = screen->display->atom_net_wm_allowed_actions;
-  atoms[34] = screen->display->atom_net_wm_action_move;
-  atoms[35] = screen->display->atom_net_wm_action_resize;
-  atoms[36] = screen->display->atom_net_wm_action_shade;
-  atoms[37] = screen->display->atom_net_wm_action_stick;
-  atoms[38] = screen->display->atom_net_wm_action_maximize_horz;
-  atoms[39] = screen->display->atom_net_wm_action_maximize_vert;
-  atoms[40] = screen->display->atom_net_wm_action_change_desktop;
-  atoms[41] = screen->display->atom_net_wm_action_close;
-  atoms[42] = screen->display->atom_net_wm_state_above;
-  atoms[43] = screen->display->atom_net_wm_state_below;
-  atoms[44] = screen->display->atom_net_startup_id;
+  atoms[21] = screen->display->atom_net_wm_icon_name;
+  atoms[22] = screen->display->atom_net_wm_icon;
+  atoms[23] = screen->display->atom_net_wm_icon_geometry;
+  atoms[24] = screen->display->atom_net_wm_moveresize;
+  atoms[25] = screen->display->atom_net_active_window;
+  atoms[26] = screen->display->atom_net_wm_strut;
+  atoms[27] = screen->display->atom_net_wm_state_hidden;
+  atoms[28] = screen->display->atom_net_wm_window_type_utility;
+  atoms[29] = screen->display->atom_net_wm_window_type_splash;
+  atoms[30] = screen->display->atom_net_wm_state_fullscreen;
+  atoms[31] = screen->display->atom_net_wm_ping;
+  atoms[32] = screen->display->atom_net_wm_pid;
+  atoms[33] = screen->display->atom_net_workarea;
+  atoms[34] = screen->display->atom_net_showing_desktop;
+  atoms[35] = screen->display->atom_net_desktop_layout;
+  atoms[36] = screen->display->atom_net_desktop_names;
+  atoms[37] = screen->display->atom_net_wm_allowed_actions;
+  atoms[38] = screen->display->atom_net_wm_action_move;
+  atoms[39] = screen->display->atom_net_wm_action_resize;
+  atoms[40] = screen->display->atom_net_wm_action_shade;
+  atoms[41] = screen->display->atom_net_wm_action_stick;
+  atoms[42] = screen->display->atom_net_wm_action_maximize_horz;
+  atoms[43] = screen->display->atom_net_wm_action_maximize_vert;
+  atoms[44] = screen->display->atom_net_wm_action_change_desktop;
+  atoms[45] = screen->display->atom_net_wm_action_close;
+  atoms[46] = screen->display->atom_net_wm_state_above;
+  atoms[47] = screen->display->atom_net_wm_state_below;
+  atoms[48] = screen->display->atom_net_startup_id;
   
   XChangeProperty (screen->display->xdisplay, screen->xroot,
                    screen->display->atom_net_supported,
                    XA_ATOM,
                    32, PropModeReplace, (guchar*) atoms, N_SUPPORTED);
-
-  /* Set legacy GNOME hints */
-  atoms[0] = screen->display->atom_win_layer;
-  
-  XChangeProperty (screen->display->xdisplay, screen->xroot,
-                   screen->display->atom_win_protocols,
-                   XA_ATOM,
-                   32, PropModeReplace, (guchar*) atoms, N_WIN_SUPPORTED);
   
   return Success;
 #undef N_SUPPORTED
@@ -511,7 +496,6 @@ meta_screen_new (MetaDisplay *display,
                 SubstructureRedirectMask | SubstructureNotifyMask |
                 ColormapChangeMask | PropertyChangeMask |
                 LeaveWindowMask | EnterWindowMask |
-                ButtonPressMask | ButtonReleaseMask |
                 KeyPressMask | KeyReleaseMask |
                 FocusChangeMask | StructureNotifyMask |
 		attr.your_event_mask);
@@ -1204,39 +1188,6 @@ meta_screen_ensure_workspace_popup (MetaScreen *screen)
   /* don't show tab popup, since proper space isn't selected yet */
 }
 
-/* Focus top window on active workspace */
-void
-meta_screen_focus_top_window (MetaScreen *screen,
-                              MetaWindow *not_this_one)
-{
-  MetaWindow *window;
-
-  if (not_this_one)
-    meta_topic (META_DEBUG_FOCUS,
-                "Focusing top window excluding %s\n", not_this_one->desc);
-  
-  window = meta_stack_get_default_focus_window (screen->stack,
-                                                screen->active_workspace,
-                                                not_this_one);
-
-  /* FIXME I'm a loser on the CurrentTime front */
-  if (window)
-    {
-      meta_topic (META_DEBUG_FOCUS,
-                  "Focusing top window %s\n", window->desc);
-
-      meta_window_focus (window, meta_display_get_current_time (screen->display));
-
-      /* Also raise the window if in click-to-focus */
-      if (meta_prefs_get_focus_mode () == META_FOCUS_MODE_CLICK)
-        meta_window_raise (window);
-    }
-  else
-    {
-      meta_topic (META_DEBUG_FOCUS, "No top window to focus found\n");
-    }
-}
-
 void
 meta_screen_focus_mouse_window (MetaScreen  *screen,
                                 MetaWindow  *not_this_one)
@@ -1287,16 +1238,6 @@ meta_screen_focus_mouse_window (MetaScreen  *screen,
     }
 }
 
-void
-meta_screen_focus_default_window (MetaScreen *screen,
-                                  MetaWindow *not_this_one)
-{
-  if (meta_prefs_get_focus_mode () == META_FOCUS_MODE_CLICK)
-    meta_screen_focus_top_window (screen, not_this_one);
-  else
-    meta_screen_focus_mouse_window (screen, not_this_one);
-}
-
 const MetaXineramaScreenInfo*
 meta_screen_get_xinerama_for_rect (MetaScreen    *screen,
 				   MetaRectangle *rect)
@@ -1346,22 +1287,151 @@ meta_screen_get_xinerama_for_window (MetaScreen *screen,
   return meta_screen_get_xinerama_for_rect (screen, &window_rect);
 }
 
-gboolean
-meta_screen_window_intersects_xinerama (MetaScreen *screen,
-                                        MetaWindow *window,
-                                        int         which_xinerama)
+const MetaXineramaScreenInfo* 
+meta_screen_get_xinerama_neighbor (MetaScreen         *screen,
+                                   int                 which_xinerama,
+                                   MetaScreenDirection direction)
 {
-  MetaRectangle window_rect;
-  MetaRectangle dest, screen_rect;
+  MetaXineramaScreenInfo* input = screen->xinerama_infos + which_xinerama;
+  MetaXineramaScreenInfo* current;
+  int i;
+
+  for (i = 0; i < screen->n_xinerama_infos; i++)
+    {
+      current = screen->xinerama_infos + i;
+
+      if (((direction == META_SCREEN_RIGHT) && 
+           (current->x_origin == input->x_origin + input->width) &&
+           (current->y_origin >= input->y_origin) &&
+           (current->y_origin <= input->y_origin+input->height)) ||
+          ((direction == META_SCREEN_LEFT) && 
+           (input->x_origin == current->x_origin + current->width) &&
+           (current->y_origin >= input->y_origin) &&
+           (current->y_origin <= input->y_origin + input->height)) ||
+          ((direction == META_SCREEN_UP) && 
+           (input->y_origin == current->y_origin + current->height) &&
+           (current->x_origin >= input->x_origin) &&
+           (current->x_origin <= input->x_origin + input->width)) ||
+          ((direction == META_SCREEN_DOWN) && 
+           (current->y_origin == input->y_origin + input->height) &&
+           (current->x_origin >= input->x_origin) &&
+           (current->x_origin <= input->x_origin + input->width)))
+        {
+          return current;
+        }
+    }
   
-  meta_window_get_outer_rect (window, &window_rect);
+  return NULL;
+}
+
+void
+meta_screen_get_natural_xinerama_list (MetaScreen *screen,
+                                       int**       xineramas_list,
+                                       int*        n_xineramas)
+{
+  const MetaXineramaScreenInfo* current;
+  const MetaXineramaScreenInfo* tmp;
+  GQueue* xinerama_queue;
+  int* visited;
+  int cur = 0;
+  int i;
+
+  *n_xineramas = screen->n_xinerama_infos;
+  *xineramas_list = g_new (int, screen->n_xinerama_infos);
+
+  /* we calculate a natural ordering by which to choose xineramas for
+   * window placement.  We start at the current xinerama, and perform
+   * a breadth-first search of the xineramas starting from that
+   * xinerama.  We choose preferentially left, then right, then down,
+   * then up.  The visitation order produced by this traversal is the
+   * natural xinerama ordering.
+   */
+
+  visited = g_new (int, screen->n_xinerama_infos);
+  for (i = 0; i < screen->n_xinerama_infos; i++)
+    {
+      visited[i] = FALSE;
+    }
+
+  current = meta_screen_get_current_xinerama (screen);
+  xinerama_queue = g_queue_new ();
+  g_queue_push_tail (xinerama_queue, (gpointer) current);
+  visited[current->number] = TRUE;
+
+  while (!g_queue_is_empty (xinerama_queue))
+    {
+      current = (const MetaXineramaScreenInfo*) 
+        g_queue_pop_head (xinerama_queue);
+
+      (*xineramas_list)[cur++] = current->number;
+
+      /* enqueue each of the directions */
+      tmp = meta_screen_get_xinerama_neighbor (screen,
+                                               current->number,
+                                               META_SCREEN_LEFT);
+      if (tmp && !visited[tmp->number])
+        {
+          g_queue_push_tail (xinerama_queue,
+                             (MetaXineramaScreenInfo*) tmp);
+          visited[tmp->number] = TRUE;
+        }
+      tmp = meta_screen_get_xinerama_neighbor (screen,
+                                               current->number,
+                                               META_SCREEN_RIGHT);
+      if (tmp && !visited[tmp->number])
+        {
+          g_queue_push_tail (xinerama_queue,
+                             (MetaXineramaScreenInfo*) tmp);
+          visited[tmp->number] = TRUE;
+        }
+      tmp = meta_screen_get_xinerama_neighbor (screen,
+                                               current->number,
+                                               META_SCREEN_UP);
+      if (tmp && !visited[tmp->number])
+        {
+          g_queue_push_tail (xinerama_queue,
+                             (MetaXineramaScreenInfo*) tmp);
+          visited[tmp->number] = TRUE;
+        }
+      tmp = meta_screen_get_xinerama_neighbor (screen,
+                                               current->number,
+                                               META_SCREEN_DOWN);
+      if (tmp && !visited[tmp->number])
+        {
+          g_queue_push_tail (xinerama_queue,
+                             (MetaXineramaScreenInfo*) tmp);
+          visited[tmp->number] = TRUE;
+        }
+    }
+
+  /* in case we somehow missed some set of xineramas, go through the
+   * visited list and add in any xineramas that were missed
+   */
+  for (i = 0; i < screen->n_xinerama_infos; i++)
+    {
+      if (visited[i] == FALSE)
+        {
+          (*xineramas_list)[cur++] = i;
+        }
+    }
+
+  g_free (visited);
+  g_queue_free (xinerama_queue);
+}
+
+gboolean
+meta_screen_rect_intersects_xinerama (MetaScreen    *screen,
+                                      MetaRectangle *rect,
+                                      int            which_xinerama)
+{
+  MetaRectangle dest, screen_rect;
   
   screen_rect.x = screen->xinerama_infos[which_xinerama].x_origin;
   screen_rect.y = screen->xinerama_infos[which_xinerama].y_origin;
   screen_rect.width = screen->xinerama_infos[which_xinerama].width;
   screen_rect.height = screen->xinerama_infos[which_xinerama].height;
   
-  if (meta_rectangle_intersect (&screen_rect, &window_rect, &dest))
+  if (meta_rectangle_intersect (&screen_rect, rect, &dest))
     return TRUE;
 
   return FALSE;
@@ -1991,6 +2061,10 @@ meta_screen_resize_func (MetaScreen *screen,
                          MetaWindow *window,
                          void       *user_data)
 {
+  if (window->struts)
+    {
+      meta_window_update_struts (window);
+    }
   meta_window_queue_move_resize (window);
 }
 
@@ -2045,6 +2119,31 @@ queue_windows_showing (MetaScreen *screen)
   g_slist_free (windows);
 }
 
+void
+meta_screen_minimize_all_except (MetaScreen *screen,
+                                 MetaWindow *keep)
+{
+  GSList *windows;
+  GSList *tmp;
+  
+  windows = meta_display_list_windows (screen->display);
+  
+  tmp = windows;
+  while (tmp != NULL)
+    {
+      MetaWindow *w = tmp->data;
+      
+      if (w->screen == screen  &&
+          w->has_minimize_func &&
+	  w != keep)
+	meta_window_minimize (w);
+      
+      tmp = tmp->next;
+    }
+  
+  g_slist_free (windows);
+}
+
 void
 meta_screen_show_desktop (MetaScreen *screen)
 {
@@ -2056,6 +2155,8 @@ meta_screen_show_desktop (MetaScreen *screen)
   queue_windows_showing (screen);
 
   update_showing_desktop_hint (screen);
+
+  meta_workspace_focus_top_window (screen->active_workspace, NULL);
 }
 
 void
@@ -2069,8 +2170,6 @@ meta_screen_unshow_desktop (MetaScreen *screen)
   queue_windows_showing (screen);
 
   update_showing_desktop_hint (screen);
-
-  meta_screen_focus_top_window (screen, NULL);
 }
 
 #ifdef HAVE_STARTUP_NOTIFICATION
