@@ -51,28 +51,6 @@ meta_ui_init (int *argc, char ***argv)
   if (!gtk_init_check (argc, argv))
     meta_fatal ("Unable to open X display %s\n", XDisplayName (NULL));
 
-  {
-    /* FIXME hackaround for Pango opening a separate display
-     * connection and doing a server grab while we have a grab
-     * on the primary display connection. This forces Pango to
-     * go ahead and do its font cache before we try to grab
-     * the server.
-     */  
-    PangoFontMetrics *metrics;
-    PangoLanguage *lang;
-    PangoContext *context;
-    PangoFontDescription *font_desc;
-
-    context = gdk_pango_context_get ();
-    lang = gtk_get_default_language ();
-    font_desc = pango_font_description_from_string ("Sans 12");
-    metrics = pango_context_get_metrics (context, font_desc, lang);    
-    
-    pango_font_metrics_unref (metrics);
-    pango_font_description_free (font_desc);
-    g_object_unref (G_OBJECT (context));
-  }
-
   meta_stock_icons_init ();
 }
 
@@ -174,21 +152,78 @@ meta_ui_get_frame_geometry (MetaUI *ui,
                             left_width, right_width);
 }
 
-
-void
-meta_ui_add_frame (MetaUI *ui,
-                   Window  xwindow)
+Window
+meta_ui_create_frame_window (MetaUI *ui,
+			     Display *xdisplay,
+			     gint x,
+			     gint y,
+			     gint width,
+			     gint height,
+			     gint screen_no)
 {
-  meta_frames_manage_window (ui->frames, xwindow);
+  GdkDisplay *display = gdk_x11_lookup_xdisplay (xdisplay);
+  GdkScreen *screen = gdk_display_get_screen (display, screen_no);
+  GdkWindowAttr attrs;
+  gint attributes_mask;
+  GdkWindow *window;
+  
+  /* Default depth/visual handles clients with weird visuals; they can
+   * always be children of the root depth/visual obviously, but
+   * e.g. DRI games can't be children of a parent that has the same
+   * visual as the client.
+   */
+
+  attrs.title = NULL;
+
+  /* frame.c is going to replace the event mask immediately, but
+   * we still have to set it here to let GDK know what it is.
+   */
+  attrs.event_mask =
+    GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
+    GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK |
+    GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK | GDK_FOCUS_CHANGE_MASK;
+  attrs.x = x;
+  attrs.y = y;
+  attrs.wclass = GDK_INPUT_OUTPUT;
+  attrs.visual = gdk_screen_get_system_visual (screen);
+  attrs.colormap = gdk_screen_get_default_colormap (screen);
+  attrs.window_type = GDK_WINDOW_CHILD;
+  attrs.cursor = NULL;
+  attrs.wmclass_name = NULL;
+  attrs.wmclass_class = NULL;
+  attrs.override_redirect = FALSE;
+
+  attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
+
+  window =
+    gdk_window_new (gdk_screen_get_root_window(screen),
+		    &attrs, attributes_mask);
+
+  gdk_window_resize (window, width, height);
+  
+  meta_frames_manage_window (ui->frames, GDK_WINDOW_XID (window), window);
+
+  return GDK_WINDOW_XID (window);
 }
 
 void
-meta_ui_remove_frame (MetaUI *ui,
-                      Window  xwindow)
+meta_ui_destroy_frame_window (MetaUI *ui,
+			      Window  xwindow)
 {
   meta_frames_unmanage_window (ui->frames, xwindow);
 }
 
+void
+meta_ui_move_resize_frame (MetaUI *ui,
+			   Window frame,
+			   int x,
+			   int y,
+			   int width,
+			   int height)
+{
+  meta_frames_move_resize_frame (ui->frames, frame, x, y, width, height);
+}
+
 void
 meta_ui_map_frame   (MetaUI *ui,
                      Window  xwindow)
@@ -312,21 +347,16 @@ meta_image_window_new (Display *xdisplay,
                        int      max_height)
 {
   MetaImageWindow *iw;
-  
+  GdkDisplay *gdisplay;
+  GdkScreen *gscreen;
+    
   iw = g_new (MetaImageWindow, 1);
   iw->window = gtk_window_new (GTK_WINDOW_POPUP);
-
-#ifdef HAVE_GTK_MULTIHEAD
-  {
-    GdkDisplay *gdisplay;
-    GdkScreen *gscreen;
     
-    gdisplay = gdk_x11_lookup_xdisplay (xdisplay);
-    gscreen = gdk_display_get_screen (gdisplay, screen_number);
-    
-    gtk_window_set_screen (GTK_WINDOW (iw->window), gscreen);
-  }
-#endif
+  gdisplay = gdk_x11_lookup_xdisplay (xdisplay);
+  gscreen = gdk_display_get_screen (gdisplay, screen_number);
+  
+  gtk_window_set_screen (GTK_WINDOW (iw->window), gscreen);
  
   gtk_widget_realize (iw->window);
   iw->pixmap = gdk_pixmap_new (iw->window->window,
@@ -413,11 +443,8 @@ get_cmap (GdkPixmap *pixmap)
       else
         {
           meta_verbose ("Using system cmap to snapshot pixmap\n");
-#ifdef HAVE_GTK_MULTIHEAD
           cmap = gdk_screen_get_system_colormap (gdk_drawable_get_screen (pixmap));
-#else
-          cmap = gdk_colormap_get_system ();
-#endif
+
           g_object_ref (G_OBJECT (cmap));
         }
     }
@@ -496,18 +523,21 @@ meta_gdk_pixbuf_get_from_pixmap (GdkPixbuf   *dest,
   else
     drawable = gdk_pixmap_foreign_new (xpixmap);
 
-  cmap = get_cmap (drawable);
+  if (drawable)
+    {
+      cmap = get_cmap (drawable);
   
-  retval = gdk_pixbuf_get_from_drawable (dest,
-                                         drawable,
-                                         cmap,
-                                         src_x, src_y,
-                                         dest_x, dest_y,
-                                         width, height);
-
+      retval = gdk_pixbuf_get_from_drawable (dest,
+                                             drawable,
+                                             cmap,
+                                             src_x, src_y,
+                                             dest_x, dest_y,
+                                             width, height);
+    }
   if (cmap)
     g_object_unref (G_OBJECT (cmap));
-  g_object_unref (G_OBJECT (drawable));
+  if (drawable)
+    g_object_unref (G_OBJECT (drawable));
 
   return retval;
 }
@@ -743,12 +773,11 @@ meta_ui_window_is_widget (MetaUI *ui,
 
   window = gdk_xid_table_lookup (xwindow);
 
-  if (window &&
-      gdk_window_get_window_type (window) != GDK_WINDOW_FOREIGN)
+  if (window)
     {
       void *user_data = NULL;
       gdk_window_get_user_data (window, &user_data);
-      return user_data != NULL;
+      return user_data != NULL && user_data != ui->frames;
     }
   else
     return FALSE;
@@ -789,9 +818,37 @@ meta_stock_icons_init (void)
       icon_set = gtk_icon_set_new_from_pixbuf (pixbuf);
       gtk_icon_factory_add (factory, items[i].stock_id, icon_set);
       gtk_icon_set_unref (icon_set);
-		
+      
       g_object_unref (G_OBJECT (pixbuf));
     }
 
   g_object_unref (G_OBJECT (factory));
 }
+
+int
+meta_ui_get_double_click_timeout (MetaUI *ui)
+{
+  GtkSettings *settings;
+  int timeout;
+
+  settings = gtk_widget_get_settings (GTK_WIDGET (ui->frames));
+
+  timeout = 250;
+  g_object_get (G_OBJECT (settings), "gtk-double-click-time", &timeout, NULL);
+
+  return timeout;
+}
+
+int
+meta_ui_get_drag_threshold (MetaUI *ui)
+{
+  GtkSettings *settings;
+  int threshold;
+
+  settings = gtk_widget_get_settings (GTK_WIDGET (ui->frames));
+
+  threshold = 8;
+  g_object_get (G_OBJECT (settings), "gtk-dnd-drag-threshold", &threshold, NULL);
+
+  return threshold;
+}
