@@ -64,6 +64,8 @@ static gboolean is_verbose = FALSE;
 static gboolean is_debugging = FALSE;
 static gboolean replace_current = FALSE;
 static int no_prefix = 0;
+
+#ifdef WITH_VERBOSE_MODE
 static FILE* logfile = NULL;
 
 static void
@@ -104,12 +106,13 @@ ensure_logfile (void)
         }
       else
         {
-          g_print (_("Opened log file %s\n"), filename);
+          g_printerr (_("Opened log file %s\n"), filename);
         }
       
       g_free (filename);
     }
 }
+#endif
 
 gboolean
 meta_is_verbose (void)
@@ -120,6 +123,11 @@ meta_is_verbose (void)
 void
 meta_set_verbose (gboolean setting)
 {
+#ifndef WITH_VERBOSE_MODE
+  if (setting)
+    meta_fatal (_("Metacity was compiled without support for verbose mode\n"));
+#endif
+  
   if (setting)
     ensure_logfile ();
   
@@ -158,19 +166,23 @@ utf8_fputs (const char *str,
             FILE       *f)
 {
   char *l;
-
+  int retval;
+  
   l = g_locale_from_utf8 (str, -1, NULL, NULL, NULL);
 
   if (l == NULL)
-    fputs (str, f); /* just print it anyway, better than nothing */
+    retval = fputs (str, f); /* just print it anyway, better than nothing */
   else
-    fputs (l, f);
+    retval = fputs (l, f);
 
   g_free (l);
+
+  return retval;
 }
 
+#ifdef WITH_VERBOSE_MODE
 void
-meta_debug_spew (const char *format, ...)
+meta_debug_spew_real (const char *format, ...)
 {
   va_list args;
   gchar *str;
@@ -195,9 +207,11 @@ meta_debug_spew (const char *format, ...)
   
   g_free (str);
 }
+#endif /* WITH_VERBOSE_MODE */
 
+#ifdef WITH_VERBOSE_MODE
 void
-meta_verbose (const char *format, ...)
+meta_verbose_real (const char *format, ...)
 {
   va_list args;
   gchar *str;
@@ -222,7 +236,9 @@ meta_verbose (const char *format, ...)
   
   g_free (str);
 }
+#endif /* WITH_VERBOSE_MODE */
 
+#ifdef WITH_VERBOSE_MODE
 static const char*
 topic_name (MetaDebugTopic topic)
 {
@@ -262,17 +278,23 @@ topic_name (MetaDebugTopic topic)
       return "STARTUP";
     case META_DEBUG_PREFS:
       return "PREFS";
+    case META_DEBUG_GROUPS:
+      return "GROUPS";
+    case META_DEBUG_RESIZING:
+      return "RESIZING";
+    case META_DEBUG_SHAPES:
+      return "SHAPES";
     }
 
-  return "Window manager";
+  return "WM";
 }
 
 static int sync_count = 0;
 
 void
-meta_topic (MetaDebugTopic topic,
-            const char *format,
-            ...)
+meta_topic_real (MetaDebugTopic topic,
+                 const char *format,
+                 ...)
 {
   va_list args;
   gchar *str;
@@ -304,6 +326,7 @@ meta_topic (MetaDebugTopic topic,
   
   g_free (str);
 }
+#endif /* WITH_VERBOSE_MODE */
 
 void
 meta_bug (const char *format, ...)
